#<profile4xi profile = "02" /> 
# -------------------------------------------------------------------------------------------------
#<program4xi program = "server4xi.py">
#<abstract4xi abstract = "provides Server4Xi class"/>
#<refer4xi reference = "managers.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'Server4Xi' ] 
#"
#/>

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
try:                 # system ones
    import os
    import sys
    import signal
except:
    pass
try:                 # threading ones       
    import threading                
    import _thread
    import _winapi
       
except:
    pass
try:                 # most-common ones
    import weakref
    from traceback import format_exc
except:
    pass
try:                 # rare ones
    import array
    import queue          
    import copyreg  
except:
    pass 
try:                 # imported-as ones 
    pass          
except:
    pass            
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
try:                             # Binary ones
    import _multiprocessing
except:
    pass
try:                             # Python ones
    from multiprocessing.managers import SyncManager
except:
    print("server4xi.py *** failed in from multiprocessing.managers import SyncManager ***")
try:                             
    from multiprocessing.managers import Server
except:
    print("server4xi.py *** failed in multiprocessing.managers import Server ***")      
#"
#/>

# -------------------------------------------------------------------------------------------------
# DragonXi imports                               
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi

from active_children4xi import active_children4xi
from answer_challenge4xi import answer_challenge4xi
from authenticationstring4xi import AuthenticationString4Xi

from constant_managers4xi import get_backlog_manager4xi
from constant_managers4xi import get_stop_manager4xi
from constant_managers4xi import get_traceback_manager4xi
from constant_managers4xi import get_error_manager4xi
from constant_managers4xi import get_proxy_manager4xi
from constant_managers4xi import get_return_manager4xi
from constant_managers4xi import get_unserializable_manager4xi
from constant_managers4xi import get_return_manager4xi
from constant_managers4xi import get_shutdown_manager4xi

from current_process4xi import set_manager_server_current_process4xi
from current_process4xi import get_manager_server_current_process4xi

from deliver_challenge4xi import deliver_challenge4xi

from examine4xi import examine4xi

from listener_client4xi import listener_client4xi

from popen4xi import Popen4Xi
from public_methods4xi import public_methods4xi

from reason_object4xi import reason_object4xi

from sense_object4xi import sense_object4xi

from token4xi import Token4Xi
from _run_finalizers4xi import _run_finalizers4xi

from print4xi import print4xi
from time4xi import time4xi
#"
#/>


#
# =================================================================================================
#<class4xi class = "Server4Xi" base = "object" child = "None" 
#  synopsis = "Server4Xi class is run in a process controlled by a manager object"
#  usage = "
#   managers.py
#       BaseManager:
#           _Server = Server    #class variable
#           get_server:
#               return Server(self._registry, self._address, self._authkey, self._serializer) 
#           _run_server:
#               server = cls._Server(registry, address, authkey, serializer), where cls is parameter
#
#   basemanager4xi.py:
#
#   syncmanager4xi:
#       SyncManager4Xi
#           get_server:
#               return Server4Xi(self._registry, self._address, self._authkey, self._serializer)
#           _run_server:
#               server2xi = cls._Server(registry, address, authkey, serializer), where cls is parameter
#"
#>
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# =================================================================================================
class Server4Xi(object):
    '''
    Server being run in a process controlled by a manager object
    '''
    
    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi expose = "
    _exposed_ = ('decref', 'incref', 'accept_connection', 'get_methods',
                 'create', 'shutdown',
                 'number_of_objects', 'debug_info', 'dummy',                 
                 'fallback_repr', 'fallback_str', 'fallback_getvalue',                 
                 'serve_client', 'handle_request', 'serve_forever','accepter',
                 'sense', 'reason', 'ace')          
    #"
    #/>
        
    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    public = ['shutdown', 'create', 'accept_connection', 'get_methods',
              'debug_info', 'number_of_objects', 'dummy', 'incref', 'decref', 'sense', 'reason', 'ace']
    #"
    #/>
                         
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-Server4X" usage = "initializes with Server4Xi">                       
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __init__(self, registry, address, authkey, serializer):
        '''
        initializes Server4Xi  
        '''


        # -----------------------------------------------------------------------------------------    
        # (Step 1/12)
        # -----------------------------------------------------------------------------------------
                         
        class2print = "Server4Xi"
        method2print = "__init__"        
        
        message2print = "{(Step 1/12a) started" + "; init-of-Server4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        # 01:48:40 02/12/13 Pacific Standard Time
        
        message2print = "{(Step 1/12b) borning" + "; server2xi-of-Server4Xi-borning: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        # 01:48:40 02/12/13 Pacific Standard Time
               
        message2print = "{(Step 1/12d) inheriting" + "; object-inheriting-by-server2xi-of-Server4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'object'>
        
        message2print = "{(Step 1/12e) sharing" + "; public-sharing-by-server2xi-of-Server4Xi: " + str(Server4Xi.public)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                      
        # ['shutdown', 'create', 'accept_connection', 'get_methods', 'debug_info', 'number_of_objects', 'dummy', 'incref', 'decref', 'sense', 'reason', 'ace']
               
                                
        message2print = "{(Step 1/12f) passed" + "; registry-passed-to-init-of-Server4Xi: " + str(registry) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
        # {}                   

        # -----------------------------------------------------------------------------------------    
        # 
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 1/12g) passed" + "; address-passed-to-init-of-Server4Xi: " + str(address) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
        # ('localhost', 8080)
        
        if address is None:
            message2print = "{(Step 1/12h) warned" + "; address-warned-by-init-of-Server4Xi: " + str(address)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
         
        message2print = "{(Step 1/12i) passed" + "; authkey-passed-to-init-of-Server4Xi: " + str(authkey) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
        # b'\x9b/jp!8\xd07\xcb\xa4\xdc++{\xe4\x87\xf0A\x9a\xdd\xa3\xc4\x82\xf6\x13)\x91\xb8\xc4\xd6\x06\x0e'
        
        # -----------------------------------------------------------------------------------------    
        # 
        # -----------------------------------------------------------------------------------------
     
        if authkey is None:
            authkey = os.urandom(32)
            
            message2print = "{(Step 1/12i2) set" + "; authkey-set-by-init-of-Server4Xi to: " + str(authkey)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                        
        message2print = "{(Step 1/12j) passed" + "; serializer-passed-to-init-of-Server4Xi: " + str(serializer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
        # pickle
                              
        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            
            message2print = "{(StepOriginal) passing" + "; registry-passing-to-Server-by-init-of-Server4Xi: " + str(registry) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # {}
            
            message2print = "{(StepOriginal) passing" + "; address-passing-to-Server-by-init-of-Server4Xi: " + str(address) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # ('localhost', 8080)
            
            message2print = "{(StepOriginal) passing" + "; authkey-passing-to-Server-by-init-of-Server4Xi: " + str(authkey) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # b'\x9b/jp!8\xd07\xcb\xa4\xdc++{\xe4\x87\xf0A\x9a\xdd\xa3\xc4\x82\xf6\x13)\x91\xb8\xc4\xd6\x06\x0e'                         
            
            message2print = "{(StepOriginal) passing" + "; serializer-passing-to-Server-by-init-of-Server4Xi: " + str(serializer) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # pickle                   
                   
            Server.__init__(self, registry, address, authkey, serializer) 
            
            # message2print = "{(StepOriginal) performed" + "; Server-init-performed-by-init-of-Server4Xi: " + str(Server.__init__) + "; time: " + time4xi() + "}"
            # print4xi(class2print, method2print, message2print)             
            # <function Server.__init__ at 0x0000000002C5DC80>                
                              
        else:
            
            # -----------------------------------------------------------------------------------------    
            # (Step 2/12)
            # -----------------------------------------------------------------------------------------                 
            message2print = "{(Step 2/12a) passing" + "; authkey-passing-to-isinstance-by-init-of-Server4Xi: " + str() + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                            
            message2print = "{(Step 2/12b) passing" + "; bytes-passing-to-isinstance-by-init-of-Server4Xi: " + str(bytes) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                              

            # -----------------------------------------------------------------------------------------    
            # 
            # -----------------------------------------------------------------------------------------
                     
            if isinstance(authkey, bytes):
            
                message2print = "{(Step 2/12c1) asserted" + "; authkey-asserted-by-init-of-Server4Xi: " + str(authkey) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)      
            else:
               
                message2print = "{(Step 2/12c2) warned" + "; authkey-warned-by-init-of-Server4Xi: " + " ** " + str(authkey)  + " ** " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             

                authkey = os.urandom(32)
                                                
                message2print = "{(Step 2/12c3) set" + "; authkey-set-by-init-of-Server4Xi to: " + str(authkey)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            # -----------------------------------------------------------------------------------------    
            # (Step 3/12)
            # -----------------------------------------------------------------------------------------                 
            
            self.registry = registry
            
            message2print = "{(Step 3/12) set" + "; registry-of-server2xi-set-by-init-of-Server4Xi to: " + str(self.registry)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                            
                            
                    
            
            # -----------------------------------------------------------------------------------------    
            # (Step 4/12)
            # -----------------------------------------------------------------------------------------                 
            
            self.authkey = authkey
            
            
            message2print = "{(Step 4/12) set" + "; authkey-of-server2xi-set-by-init-of-Server4Xi to: " + str(self.authkey)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
  
                    
            # -----------------------------------------------------------------------------------------    
            # (Step 5/12)
            # -----------------------------------------------------------------------------------------                 
            ack2listener = False
            nack2listener = True
            
            ack2client = False
            nack2client = True
            
            try:
                Listener, Client = listener_client4xi[serializer]
      
                message2print = "{(Step 5/12a1) set" + "; Listener-set-by-init-of-Server4Xi to: " + str(Listener)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                
                ack2listener = True
                
                message2print = "{(Step 5/12a2) set" + "; ack2listener-set-by-init-of-Server4Xi to: " + str(ack2listener)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                

                message2print = "{(Step 5/12b1) set" + "; Client-set-by-init-of-Server4Xi to: " + str(Client)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
                ack2client = True

                message2print = "{(Step 5/12b2) set" + "; ack2client-set-by-init-of-Server4Xi to: " + str(ack2client)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                   
            except:
                message2print = "{(Step 5/12c1) yielded" + "; nack2listener-yielded-by-init-of-Server4Xi: " + str(nack2listener)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 

                message2print = "{(Step 5/12c2) yielded" + "; nack2client-yielded-by-init-of-Server4Xi: " + str(nack2client)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                
                                            
            # -----------------------------------------------------------------------------------------    
            # (Step 6/12)
            # -----------------------------------------------------------------------------------------                 
            # do authentication later
            ack2objectlistener = False
            nack2objectlistener = True
            
            if ack2listener is True:
                try:
                    self.listener = Listener(address=address, backlog=get_backlog_manager4xi()) #dragonized backlog=5
                    
                    
                    message2print = "{(Step 6/12a) set" + "; listener-of-server2xi-set-by-init-of-Server4Xi to: " + str(self.listener)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
           
                    ack2objectlistener = True
                    
                    message2print = "{(Step 6/12b) set" + "; ack2objectlistener-set-by-init-of-Server4Xi to: " + str(ack2objectlistener)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                except:
                    message2print = "{(Step 6/12c) yielded" + "; nack2objectlistener-yielded-by-init-of-Server4Xi: " + str(nack2objectlistener)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                    
            else:
                message2print = "{(Step 6/12d) yielded" + "; nack2objectlistener-yielded-by-init-of-Server4Xi: " + str(nack2objectlistener)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                     
            
            # -----------------------------------------------------------------------------------------    
            # (Step 7/12)
            # -----------------------------------------------------------------------------------------          
            ack2address = False
            nack2address = True
            
            if ack2objectlistener is True:
                try:            
                    self.address = self.listener.address
            
                    
                    message2print = "{(Step 7/12a) set" + "; address-of-server2xi-set-by-init-of-Server4Xi to: " + str(self.address)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    
                    ack2address = True
                    
                    message2print = "{(Step 7/12b) set" + "; ack2address-set-by-init-of-Server4Xi to: " + str(ack2address)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                except:
                    message2print = "{(Step 6/12c) yielded" + "; nack2address-yielded-by-init-of-Server4Xi: " + str(nack2address)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                    
            else:
                message2print = "{(Step 6/12d) yielded" + "; nack2address-yielded-by-init-of-Server4Xi: " + str(nack2address)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                     
            
                self.address = None
                
            # -----------------------------------------------------------------------------------------    
            # (Step 8/12)
            # -----------------------------------------------------------------------------------------          
            self.id_to_obj = {0: (None, ())}
            
            
            message2print = "{(Step 8/12) set" + "; id-to-object-of-server2xi-set-by-init-of-Server4Xi to: " + str(self.id_to_obj)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
          
            
            # -----------------------------------------------------------------------------------------    
            # (Step 9/12)
            # -----------------------------------------------------------------------------------------          
            self.id_to_refcount = {}
            
            
            message2print = "{(Step 9/12) set" + "; id-to-refcount-of-server2xi-set-by-init-of-Server4Xi to: " + str(self.id_to_refcount)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
  
            
            # -----------------------------------------------------------------------------------------    
            # (Step 10/12)
            # -----------------------------------------------------------------------------------------          
            self.mutex = threading.RLock() 
            
            message2print = "{(Step 10/12) set" + "; mutex-of-server2xi-set-by-init-of-Server4Xi to: " + str(self.mutex)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
      
                 
            
            # -----------------------------------------------------------------------------------------    
            # (Step 11/12)
            # -----------------------------------------------------------------------------------------          
            self.stop = 0       
            
            
            message2print = "{(Step 11/12) set" + "; stop-of-server2xi-set-by-init-of-Server4Xi to: " + str(self.stop)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
           
            
            # -----------------------------------------------------------------------------------------    
            # (Step 12/12)
            # -----------------------------------------------------------------------------------------                 
            
            message2print = "{(Step 12/12a) activated" + "; server2xi-of-Server4Xi-activated: " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                
            message2print = "{(Step 12/13b) inherited" + "; object-inherited-by-server2xi-of-Server4Xi: " + str(object)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
        
            message2print = "{(Step 12/13c) shared" + "; public-shared-by-server2xi-of-Server4Xi: " + str(Server4Xi.public)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                      
                      
        message2print = "{(Step 12/12d) returning" + "; init-of-Server4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #  02:02:43 02/12/13 Pacific Standard Time 
                                    
        return                
                                        

    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "serve-forever-of-Server4X" usage = "
    #    _run_server(cls, registry, address, authkey, serializer, writer) <--- in managers.py
    #    run the manager
    #         util.info('manager serving at %r', server.address)
    #         server.serve_forever()">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def serve_forever(self):
        '''
        runs the server forever
        '''


        # -----------------------------------------------------------------------------------------    
        # (Step 1/11)
        # -----------------------------------------------------------------------------------------            
        
        class2print = "Server4Xi"
        method2print = "serve_forever"
        
        message2print = "{(Step 1/4a) started" + "; serve-forever-of-Server4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            serializer="pickle"
            
            message2print = "{(StepOriginal) calling" + "; Server-serve-forever-calling-by-serve-forever-of-Server4Xi: " + str(Server.serve_forever) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       

            Server.serve_forever(self)             
        else:
            # -----------------------------------------------------------------------------------------    
            # (Step 2/4)
            # -----------------------------------------------------------------------------------------        
            set_manager_server_current_process4xi(self)                                                             #dragonized current_process()._manager_server = self
                        
            message2print = "{(Step 2/4a) performed" + "; set-manager-server-current-process4xi-performed-by-serve-forever-of-Server4Xi to: " + str(set_manager_server_current_process4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            try:
                # -----------------------------------------------------------------------------------------    
                # 
                # -----------------------------------------------------------------------------------------    
                accepter2xi = threading.Thread(target=self.accepter)
                message2print = "{(Step 2/4b) got" + "; accepter2xi-by-serve-forever-of-Server4Xi: " + str(accepter2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                    
                accepter2xi.daemon = True
                
                accepter2xi.start()
                
                message2print = "{(Step 2/4c) performed" + "; start-performed-by-serve-forever-of-Server4Xi to: " + str(accepter2xi.start)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                
                try:
                    while not self.stop_event.is_set():
                        self.stop_event.wait(1)

                except (KeyboardInterrupt, SystemExit):
                    pass      
            finally:
                # -----------------------------------------------------------------------------------------    
                # (Step 10/11)
                # -----------------------------------------------------------------------------------------    
                if sys.stdout != sys.__stdout__:
                    # resetting stdout, stderr
                    sys.stdout = sys.__stdout__
                    sys.stderr = sys.__stderr__
                
                message2print = "{(Step 10/11) calling" + "; sys-exit-calling-by-serve-forever-of-Server4Xi: " + str(sys.exit)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                sys.exit(0)
            
            # -----------------------------------------------------------------------------------------    
            # (Step 11/11)
            # -----------------------------------------------------------------------------------------                 
            message2print = "{(Step 11/11) returning" + "; serve-forever-of-Server4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            
            return
                            
    #</methodize4xi>
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "accepter-of-Server4X" usage = "
    #   managers.py
    #       Server(object):
    #           serve_forever:
    #               accepter = threading.Thread(target=self.accepter)
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def accepter(self):
        '''
        target function for method serve_forever to threading
        '''
        # -----------------------------------------------------------------------------------------    
        # (Step 1/2)
        # -----------------------------------------------------------------------------------------            
        class2print = "Server4Xi"
        method2print = "accepter"
        
        message2print = "{(Step 1/2a) started" + "; accepter-of-Server4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # -----------------------------------------------------------------------------------------    
        # (Step 2/2)
        # -----------------------------------------------------------------------------------------                
        while True:
            try:
                c2xi = self.listener.accept()
                
                message2print = "{(Step 2/4a) got" + "; c2xi-got-by-accepter-of-Server4Xi: " + str(c2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                 
            except (OSError, IOError):
                continue
                
            t2xi = threading.Thread(target=self.handle_request, args=(c2xi,))
            
            message2print = "{(Step 2/4b) got" + "; t2xi-got-by-accepter-of-Server4Xi: " + str(t2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            
            t2xi.daemon = True
            
            message2print = "{(Step 2/4c) starting" + "; t2xi-starting-by-accepter-of-Server4Xi: " + str(t2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            
            t2xi.start() 
            
                          
    #</methodize4xi>
                
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "handle-request-of-Server4Xi" usage = "
    #     serve_forever(self) funtion <--- in managers.py
    #         t = threading.Thread(target=self.handle_request, args=(c,))">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>    
    # ---------------------------------------------------------------------------------------------
    def handle_request(self, c):
        '''
        handles a new connection
        '''


        # -----------------------------------------------------------------------------------------    
        # (Step 1/15)
        # -----------------------------------------------------------------------------------------            

        class2print = "Server4Xi"
        method2print = "handle_request"
        
        message2print = "{(Step 1/15a) started" + "; handle-request-of-Server4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
                
        message2print = "{(Step 1/15b) passed" + "; connection-passed-to-handle-request-of-Server4Xi: " + str(c) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
      
        message2print = "{(Step 1/15c) abbreviating" + "; connection-abbreviating-by-handle-request-of-Server4Xi: " + "c" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 

        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            serializer="pickle"
            
            message2print = "{(StepOriginal) passing" + "; connection-passing-to-Server-handle-request-by-handle-request-of-Server4Xi: " + str(c) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            
                            
            message2print = "{(StepOriginal) calling" + "; Server-handle-request-calling-by-handle-request-of-Server4Xi: " + str(Server.handle_request) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
                            
            Server.handle_request(self, c)
                         
        else:

            # -----------------------------------------------------------------------------------------    
            #I (Step 2/15)
            # -----------------------------------------------------------------------------------------                 
            
            funcname2xi = result2xi = request2xi = None
            
            message2print = "{(Step 2/15a) set" + "; funcname2xi-set-by-handle-request-of-Server4Xi to: " + str(funcname2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                            
            
            message2print = "{(Step 2/15b) set" + "; result2xi-set-by-handle-request-of-Server4Xi to: " + str(result2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                              
            
            message2print = "{(Step 2/15c) set" + "; request2xi-set-by-handle-request-of-Server4Xi to: " + str(request2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
              
            
            
            # -----------------------------------------------------------------------------------------    
            #I
            # -----------------------------------------------------------------------------------------                 
            
            try:
                
                # -----------------------------------------------------------------------------------------    
                #II (Step 3/15)
                # -----------------------------------------------------------------------------------------    
                message2print = "{(Step 3/15a) passing" + "; connection-passing-to-deliver-challenge4xi-by-handle-request-of-Server4Xi: " + str(c) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                                
                message2print = "{(Step 3/15b) passing" + "; authkey-of-server2xi-passing-to-deliver-challenge4xi-by-handle-request-of-Server4Xi: " + str(self.authkey) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                 
                
                deliver_challenge4xi(c, self.authkey)
                
                message2print = "{(Step 3/15b) performed" + "; deliver-challenge4xi-performed-by-handle-request-of-Server4Xi: " + str(deliver_challenge4xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                                
           
                
                # -----------------------------------------------------------------------------------------    
                #II (Step 4/15)
                # -----------------------------------------------------------------------------------------    
                message2print = "{(Step 4/15a) passing" + "; connection-passing-to-answer-challenge4xi-by-handle-request-of-Server4Xi: " + str(c) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                                
                message2print = "{(Step 4/15b) passing" + "; authkey-of-server2xi-passing-to-answer-challenge4xi-by-handle-request-of-Server4Xi: " + str(self.authkey) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                                    
                
                answer_challenge4xi(c, self.authkey)
                
                message2print = "{(Step 4/15c) performed" + "; answer_challenge4xi-performed-by-handle-request-of-Server4Xi: " + str(answer_challenge4xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                                
                
                
                # -----------------------------------------------------------------------------------------    
                #II (Step 5/15)
                # -----------------------------------------------------------------------------------------    
                message2print = "{(Step 5/15a) calling" + "; connection-recv-calling-by-handle-request-of-Server4Xi: " + str(c.recv) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)       
                                

                request2xi = c.recv()
                
                message2print = "{(Step 5/15b) got" + "; request2xi-got-by-handle-request-of-Server4Xi: " + str(request2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                                          
         
                # -----------------------------------------------------------------------------------------    
                #II (Step 6/15)
                # -----------------------------------------------------------------------------------------    
                  
                ignore2xi, funcname2xi, args, kwds = request2xi
                
                message2print = "{(Step 6/15a) set" + "; ignore2xi-set-by-handle-request-of-Server4Xi to: " + str(ignore2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                
                
                message2print = "{(Step 6/15b) set" + "; funcname2xi-set-by-handle-request-of-Server4Xi to: " + str(funcname2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                
                message2print = "{(Step 6/15c) set" + "; args-set-by-handle-request-of-Server4Xi to: " + str(args)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                
                message2print = "{(Step 6/15d) set" + "; kwds-set-by-handle-request-of-Server4Xi to: " + str(kwds)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                         
                
                
                # -----------------------------------------------------------------------------------------    
                #II (Step 7/15)
                # -----------------------------------------------------------------------------------------    
                  
                assert funcname2xi in self.public, '%r unrecognized' % funcname2xi
                
                message2print = "{(Step 7/15) asserted" + "; funcname2xi-asserted-by-handle-request-of-Server4Xi: " + str(funcname2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                 
                
                # -----------------------------------------------------------------------------------------    
                #II (Step 8/15)
                # -----------------------------------------------------------------------------------------    
                message2print = "{(Step 8/15a) passing" + "; funcname2x-passing-to-getattr-by-handle-request-of-Server4Xi: " + str(funcname2x) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                   
                func2xi = getattr(self, funcname2xi)
                
                message2print = "{(Step 8/15b) set" + "; func2xi-set-by-handle-request-of-Server4Xi to: " + str(func2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                
            
            # -----------------------------------------------------------------------------------------    
            #
            # -----------------------------------------------------------------------------------------                 
                                      
                            
            except Exception:
                
                # -----------------------------------------------------------------------------------------    
                #II (Step 9/15)
                # -----------------------------------------------------------------------------------------    
               
                msg2xi = (get_traceback_manager4xi(), format_exc()) #TRACEBACK_MANAGER4XI
    
                message2print = "{(Step 9/15) hit" + "; Exception-hit-by-handle-request-of-Server4Xi: " + " *** " + str(msg2xi)  + " *** " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                                    
            
            # -----------------------------------------------------------------------------------------    
            #I
            # -----------------------------------------------------------------------------------------                 
                                             
                
            else:
                
                # -----------------------------------------------------------------------------------------    
                #II (Step 10/15)
                # -----------------------------------------------------------------------------------------    
               
                try:
                
                    # -----------------------------------------------------------------------------------------    
                    #III 
                    # -----------------------------------------------------------------------------------------    
                          
                    message2print = "{(Step 10/15a) passing" + "; connection-passing-to-func2xi-by-handle-request-of-Server4Xi: " + str(c) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)            
                    
                                  
                    result2xi = func2xi(c, *args, **kwds)
                    
                    message2print = "{(Step 10/15d) got" + "; result2xi-got-by-handle-request-of-Server4Xi: " + str(result2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                                    
                    
                except Exception:
                
                    # -----------------------------------------------------------------------------------------    
                    #III
                    # -----------------------------------------------------------------------------------------    
                     
                    msg2xi = (get_traceback_manager4xi(), format_exc())# dragonized #TRACEBACK
                    
    
                    message2print = "{(Step 10/15e) hit" + "; Exception-hit-by-handle-request-of-Server4Xi: " + " *** " + str(msg2xi)  + " *** " + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                       
                    
                else:
                
                    # -----------------------------------------------------------------------------------------    
                    #III 
                    # -----------------------------------------------------------------------------------------    
                 
 
                    msg2xi = (get_return_manager4xi(), result2xi) #dragonized #RETURN
                    
                    message2print = "{(Step 10/15f) set" + "; msg2xi-set-by-handle-request-of-Server4Xi to: " + str(msg2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                                    
                                                    
            
            # -----------------------------------------------------------------------------------------    
            #I
            # -----------------------------------------------------------------------------------------                 
                          
            try:
                
                # -----------------------------------------------------------------------------------------    
                #II (Step 11/15)
                # -----------------------------------------------------------------------------------------    
                message2print = "{(Step 11/15a) passing" + "; msg2xi-passing-to-connection-send-by-handle-request-of-Server4Xi: " + str(msg2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                
                c.send(msg2xi)
                
                message2print = "{(Step 11/15b) performed" + "; connection-send-performed-by-handle-request-of-Server4Xi: " + str(c.send) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                         
                                
            
            # -----------------------------------------------------------------------------------------    
            #I
            # -----------------------------------------------------------------------------------------                 
                                         
            except Exception as e:
                
                # -----------------------------------------------------------------------------------------    
                #II (Step 12/15)
                # -----------------------------------------------------------------------------------------    
               
                try:
                
                    # -----------------------------------------------------------------------------------------    
                    #III 
                    # -----------------------------------------------------------------------------------------    
                 
                 
                    message2print = "{(Step 12/15) hit" + "; Exception-hit-by-handle-request-of-Server4Xi: " + " *** sending failed for message: " + str(msg2xi)  + " *** " + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                                        
                                                
 
                    c.send((get_traceback_manager4xi(), format_exc()))#dragonized TRACEBACK
    
                                         
                    
                except Exception:
                
                    # -----------------------------------------------------------------------------------------    
                    #III 
                    # -----------------------------------------------------------------------------------------    
                 
                 
                    pass
                
                # -----------------------------------------------------------------------------------------    
                #II (Step 13/15)
                # -----------------------------------------------------------------------------------------    
                
                message2print = "{(Step 13/15a) warned" + "; Failure-warned-by-handle-request-of-Server4Xi: " + " ** failed to send message: " + str(msg2xi)  + " ** " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)               
                
                message2print = "{(Step 13/15b) warned" + "; request2xi-warned-by-handle-request-of-Server4Xi: " + " ** failed with request: " + str(request2xi)  + " ** " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)               
                                            
                
                message2print = "{(Step 13/15c) warned" + "; exception-warned-by-handle-request-of-Server4Xi: " + " ** exception in sending: " + str(e)  + " ** " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)               
                                
                
                
                info4xi('Server4Xi method handle_request: Failure to send message: %r', msg2xi)
                info4xi(' ... request was %r', request2xi)
                info4xi(' ... exception was %r', e)
            
                
            # -----------------------------------------------------------------------------------------    
            #I (Step 14/15)
            # -----------------------------------------------------------------------------------------                 
    
            c.close()
            
            message2print = "{(Step 14/15) performed" + "; connection-close-performed-by-handle-request-of-Server4Xi: " + str(c.close) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                            
                    
            # -----------------------------------------------------------------------------------------    
            #I (Step 15/15)
            # -----------------------------------------------------------------------------------------                 
            message2print = "{(Step 15/15) returning" + "; handle-request-of-Server4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                                    
                  
            # -----------------------------------------------------------------------------------------    
            #
            # -----------------------------------------------------------------------------------------                 
                            
                                    
                   
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    # serve_client                             
    #
    # usage:
    #     accept_connection(self, c, name) <--- in managers.py
    #         self.serve_client(c)
    #<methodize4xi method = "serve-client-of-Server4Xi" usage = "handles requests from the proxies in a particular process/thread">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>    
    # ---------------------------------------------------------------------------------------------
    def serve_client(self, conn):
        '''
        handles requests from the proxies in a particular process/thread
        '''


        # -----------------------------------------------------------------------------------------    
        # (Step 1/32)
        # -----------------------------------------------------------------------------------------            
        class2print = "Server4Xi"
        method2print = "serve_client"
        
        message2print = "{(Step 1/32a) started" + "; serve-client-of-Server4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
        message2print = "{(Step 1/32b) abbreviating" + "; connection-abbreviating-by-serve-client-of-Server4Xi: " + str(conn) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        
        message2print = "{(Step 1/32c) passed" + "; connection-passed-to-serve-client-of-Server4Xi: " + str(conn) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
          
        if examine4xi(class2print, method2print):
        
            # -----------------------------------------------------------------------------------------
            # StepOriginal:        
            # ----------------------------------------------------------------------------------------- 
            message2print = "{passing" + "; connection-passing-to-Server-serve-client-by-serve-client-of-Server4Xi: " + str(conn) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            

            Server.serve_client(self, conn)           
        else:
        
            # -----------------------------------------------------------------------------------------    
            #I (Step 2/32)
            # -----------------------------------------------------------------------------------------                 
            message2print = "{(Step 2/32) checking" + "; current-thread-name-checking-by-serve-client-of-Server4Xi: " + str(threading.current_thread().name) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
                          
            
            # starting server thread to service
            
            # -----------------------------------------------------------------------------------------    
            #I (Step 3/32)
            # -----------------------------------------------------------------------------------------                 
    
            recv2xi = conn.recv
            
            message2print = "{(Step 3/32) set" + "; recv2xi-set-by-serve-client-of-Server4Xi to: " + str(recv2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                            
                                 
            
            # -----------------------------------------------------------------------------------------    
            #I (Step 4/32)
            # -----------------------------------------------------------------------------------------                 
            
            send2xi = conn.send
            
            
            message2print = "{(Step 4/32) set" + "; send2xi-set-by-serve-client-of-Server4Xi to: " + str(send2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                  
            
            # -----------------------------------------------------------------------------------------    
            #I (Step 5/32)
            # -----------------------------------------------------------------------------------------                 
            message2print = "{(Step 5/32a) using" + "; id-to-object-of-server2xi-using-by-serve-client-of-Server4Xi: " + str(self.id_to_obj)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            
            id_to_obj2xi = self.id_to_obj
            
            
            message2print = "{(Step 5/32b) set" + "; id-to-obj2xi-set-by-serve-client-of-Server4Xi to: " + str(id_to_obj2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
       
            
            # -----------------------------------------------------------------------------------------    
            #I (Step 6/32)
            # -----------------------------------------------------------------------------------------                 
   
            message2print = "{(Step 6/32) entering" + "; while-loop-entering-by-serve-client-of-Server4Xi: " + " while not self.stop " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
            while not self.stop_event.is_set():
    
                # -----------------------------------------------------------------------------------------    
                #II (Step 7/32)
                # ----------------------------------------------------------------------------------------- 
                   
                message2print = "{(Step 7/32) trying" + "; receiving-trying-by-serve-client-of-Server4Xi: " + str(recv2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)         
                                
                
                try:
    
                    # -------------------------------------------------------------------------------------    
                    #III (Step 8/32)
                    # -------------------------------------------------------------------------------------                 
                    
                
                    methodname2xi = obj2xi = None
            
                    message2print = "{(Step 8/32a) set" + "; methodname2xi-set-by-serve-client-of-Server4Xi to: " + str(methodname2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
      
            
                    message2print = "{(Step 8/32b) set" + "; obj2xi-set-by-serve-client-of-Server4Xi to: " + str(obj2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
      
                    # -------------------------------------------------------------------------------------    
                    #III (Step 9/32)
                    # -------------------------------------------------------------------------------------                 
                        
                    request2xi = recv2xi()
    
            
                    message2print = "{(Step 9/32) set" + "; request2xi-set-by-serve-client-of-Server4Xi to: " + str(request2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
    
                                    
                    # -------------------------------------------------------------------------------------    
                    #III (Step 10/32)
                    # -------------------------------------------------------------------------------------                 
                        
                    ident2xi, methodname2xi, args, kwds = request2xi
    
            
                    message2print = "{(Step 10/32a) set" + "; ident2xi-set-by-serve-client-of-Server4Xi to: " + str(ident2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
            
                    message2print = "{(Step 10/32b) set" + "; methodname2xi-set-by-serve-client-of-Server4Xi to: " + str(methodname2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
            
                    message2print = "{(Step 10/32c) set" + "; args-set-by-serve-client-of-Server4Xi to: " + str(args)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
            
                    message2print = "{(Step 10/32d) set" + "; kwds-set-by-serve-client-of-Server4Xi to: " + str(kwds)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
            
                                    
                    # -------------------------------------------------------------------------------------    
                    #III (Step 11/32)
                    # -------------------------------------------------------------------------------------                 
                        
                    obj2xi, exposed2xi, gettypeid2xi = id_to_obj2xi[ident2xi]
    
            
                    message2print = "{(Step 11/32a) set" + "; obj2xi-set-by-serve-client-of-Server4Xi to: " + str(obj2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
            
                    message2print = "{(Step 11/32b) set" + "; exposed2xi-set-by-serve-client-of-Server4Xi to: " + str(exposed2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
            
                    message2print = "{(Step 11/32c) set" + "; gettypeid2xi-set-by-serve-client-of-Server4Xi to: " + str(gettypeid2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
         
                                    
                    
                    # -------------------------------------------------------------------------------------    
                    #III (Step 12/32)
                    # -------------------------------------------------------------------------------------                 
                    
                    
                    if methodname2xi not in exposed2xi:
                    
                        # ---------------------------------------------------------------------------------    
                        #IV
                        # ---------------------------------------------------------------------------------                 
           
                        message2print = "{(Step 12/32) hit" + "; AttributeError-hit-by-serve-client-of-Server4Xi: " + " *** not exposed method = " + str(methodname2xi)  + " *** " + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                                            
                                                          
                    
                    
                        raise AttributeError(
                            'Server4Xi method serve_client: method %r of %r object is not in exposed=%r' %
                            (methodname2xi, type(obj2xi), exposed2xi)
                            )
                         
                    # -------------------------------------------------------------------------------------    
                    #III (Step 13/32)
                    # -------------------------------------------------------------------------------------                 
        
                    function2xi = getattr(obj2xi, methodname2xi)
    
            
                    message2print = "{(Step 13/32) set" + "; function2xi-set-by-serve-client-of-Server4Xi to: " + str(function2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
      
                                    
                    # -------------------------------------------------------------------------------------    
                    #III (Step 14/32)
                    # -------------------------------------------------------------------------------------                 
                    message2print = "{(Step 14/32) trying" + "; Step-14-of-32-trying-by-serve-client-of-Server4Xi: " + str(res2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)         
                                    
                            
                    try:
                        # ---------------------------------------------------------------------------------    
                        #IV (Step 15/32)
                        # ---------------------------------------------------------------------------------                 
                                   
                        
                        res2xi = function2xi(*args, **kwds)
                        
            
                        message2print = "{(Step 15/32) set" + "; res2xi-set-by-serve-client-of-Server4Xi to: " + str(res2xi)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)    
    
                    
                                        
                    except Exception as e2xi:
                                       
                        # ---------------------------------------------------------------------------------    
                        #IV (Step 16/32)
                        # ---------------------------------------------------------------------------------                     
                        msg2xi = (get_error_manager4xi(), e2xi) #dragonized ERROR_MANAGER4XI
            
                        message2print = "{(Step 16/32) set" + "; msg2xi-set-by-serve-client-of-Server4Xi to: " + str(msg2xi)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)    
                            
 
                                        
                        # ---------------------------------------------------------------------------------    
                        #
                        # ---------------------------------------------------------------------------------                 
                                     
                    else:
                        
                        # ---------------------------------------------------------------------------------    
                        #IV (Step 17/32)
                        # ---------------------------------------------------------------------------------                 
                          
                        typeid2xi = gettypeid and gettypeid.get(methodname2xi, None)
                        
            
                        message2print = "{(Step 17/32) set" + "; typeid2xi-set-by-serve-client-of-Server4Xi to: " + str(typeid2xi)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)    
      
                        
                        # ---------------------------------------------------------------------------------    
                        #IV 
                        # ---------------------------------------------------------------------------------                 
                                     
                        if typeid2xi:
                           
                            # ----------------------------------------------------------------------------    
                            #V (Step 18/32)
                            # ----------------------------------------------------------------------------                 
                                  
                            rident2xi, rexposed2xi = self.create(conn, typeid2xi, res2xi)
            
                            message2print = "{(Step 18/32a) set" + "; rident2xi-set-by-serve-client-of-Server4Xi to: " + str(rident2xi)  + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)    
            
                            message2print = "{(Step 18/32b) set" + "; rexposed2xi-set-by-serve-client-of-Server4Xi to: " + str(rexposed2xi)  + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)    
                                  
                        
                            
                            # ----------------------------------------------------------------------------    
                            #V  (Step 19/32)
                            # ----------------------------------------------------------------------------                 
                               
                            token2xi = Token4Xi(typeid2xi, self.address, rident2xi)
            
                            message2print = "{(Step 19/32) set" + "; token2xi-set-by-serve-client-of-Server4Xi to: " + str(token2xi)  + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)    
                              
                         
                                                    
                            # ----------------------------------------------------------------------------    
                            #V (Step 20/32)
                            # ----------------------------------------------------------------------------                 
                               
                            msg2xi = (get_proxy_manager4xi(), (rexposed2xi, token2xi)) #dragonized PROXY_MANAGER4XI
                            
            
                            message2print = "{(Step 20/32) set" + "; msg2xi-set-by-serve-client-of-Server4Xi to: " + str(msg2xi)  + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)    
                              
                       
                                                    
                            # ----------------------------------------------------------------------------    
                            #
                            # ----------------------------------------------------------------------------                 
                               
                        else:
                            # ----------------------------------------------------------------------------    
                            #V (Step 21/32)
                            # ----------------------------------------------------------------------------                 
                               
                            msg2xi = (get_return_manager4xi(), res2xi) #dragonized RETURN_MANAGER4XI
                            
            
                            message2print = "{(Step 21/32) set" + "; msg2xi-set-by-serve-client-of-Server4Xi to: " + str(msg2xi)  + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)    
                              
                         
                                                    
                            
                        # ---------------------------------------------------------------------------------    
                        #IV
                        # ---------------------------------------------------------------------------------                 
                 
                    # -------------------------------------------------------------------------------------    
                    #III
                    # -------------------------------------------------------------------------------------                 
        
                # -----------------------------------------------------------------------------------------    
                #II
                # -----------------------------------------------------------------------------------------    
       
                except AttributeError:
    
                    # -------------------------------------------------------------------------------------    
                    #III (Step 22/32)
                    # -------------------------------------------------------------------------------------                 
                    
                
                    if methodname2xi is None:
                            
                        # ---------------------------------------------------------------------------------    
                        #IV
                        # ---------------------------------------------------------------------------------                 
                           
                        msg2xi = (get_traceback_manager4xi(), format_exc()) #TRACEBACK_MANAGER4XI
            
                        message2print = "{(Step 22/32) set" + "; msg2xi-set-by-serve-client-of-Server4Xi to: " + str(msg2xi)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)    
                          
                     
    
                    # -------------------------------------------------------------------------------------    
                    #III
                    # -------------------------------------------------------------------------------------                 
                            
                    else:
                            
                        # ---------------------------------------------------------------------------------    
                        #IV
                        # ---------------------------------------------------------------------------------                 
                           
                        try:
                            # ----------------------------------------------------------------------------    
                            #V (Step 23/32)
                            # ----------------------------------------------------------------------------                 
                                     
                        
                            fallback_func2xi = self.fallback_mapping[methodname2xi]
                            
            
                            message2print = "{(Step 23/32) set" + "; fallback-func2xi-set-by-serve-client-of-Server4Xi to: " + str(fallback_func2xi)  + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)    
                           
                            
                            # ----------------------------------------------------------------------------    
                            #V (Step 24/32)
                            # ----------------------------------------------------------------------------                 
                                         
                            result2xi = fallback_func2xi(
                                self, conn, ident2xi, obj2xi, *args, **kwds
                                )
            
                            message2print = "{(Step 24/32) set" + "; result2xi-set-by-serve-client-of-Server4Xi to: " + str(result2xi)  + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)    
                              
                         
                                    
                            # ----------------------------------------------------------------------------    
                            #V (Step 25/32)
                            # ----------------------------------------------------------------------------                 
                                             
                            msg2xi = (get_return_manager4xi(), result) #RETURN_MANAGER4XI
                            
            
                            message2print = "{(Step 25/32) set" + "; msg2xi-set-by-serve-client-of-Server4Xi to: " + str(msg2xi)  + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)    
                              
                        
                                    
                        except Exception:
                            # ----------------------------------------------------------------------------    
                            #V (Step 26/32)
                            # ----------------------------------------------------------------------------                 
                            msg2xi = (get_traceback_manager4xi(), format_exc()) #TRACEBACK_MANAGER4XI
            
                            message2print = "{(Step 26/32a) set" + "; msg2xi-set-by-serve-client-of-Server4Xi to: " + str(msg2xi)  + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)    
      
                            message2print = "{(Step 26/32b) hit" + "; Exception-hit-by-serve-client-of-Server4Xi: " + " *** " + str(msg2xi)  + " *** " + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)
                            
                        # ---------------------------------------------------------------------------------    
                        #IV
                        # ---------------------------------------------------------------------------------                 
    
                    # -------------------------------------------------------------------------------------    
                    #III
                    # -------------------------------------------------------------------------------------                 
                            
                # -----------------------------------------------------------------------------------------    
                #II
                # -----------------------------------------------------------------------------------------    
       
                except EOFError:
                                    
                    # -------------------------------------------------------------------------------------    
                    #III (Step 27/32)
                    # -------------------------------------------------------------------------------------    
                   
                    # got EOF -- exiting thread 
                    
                    message2print = "{(Step 27/32a) hit" + "; EOFError-hit-by-serve-client-of-Server4Xi: " + " *** " + str(EOFError)  + " *** " + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                                        
                    message2print = "{(Step 27/32b) calling" + "; sys-exit-calling-by-serve-client-of-Server4Xi: " + str(sys.exit) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)       
                                    
                    
                    sys.exit(0)
    
                # -----------------------------------------------------------------------------------------    
                #II (Step 28/32)
                # -----------------------------------------------------------------------------------------    
       
                except Exception:
    
                    # -------------------------------------------------------------------------------------    
                    #III
                    # -------------------------------------------------------------------------------------                 
                          
                    msg2xi = (get_traceback_manager4xi(), format_exc()) #TRACEBACK_MANAGER4XI
                    
            
                    message2print = "{(Step 28/32a) set" + "; msg2xi-set-by-serve-client-of-Server4Xi to: " + str(msg2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
  
                    message2print = "{(Step 28/32b) hit" + "; Exception-hit-by-serve-client-of-Server4Xi: " + " *** " + str(msg2xi)  + " *** " + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                                        
                            
                # -----------------------------------------------------------------------------------------    
                #II
                # -----------------------------------------------------------------------------------------    
                                        
       
                try:
    
                    # -------------------------------------------------------------------------------------    
                    #III
                    # -------------------------------------------------------------------------------------                 
      
                    try:
                            
                        # ---------------------------------------------------------------------------------    
                        #IV
                        # ---------------------------------------------------------------------------------                 
                                        
                        # ---------------------------------------------------------------------------------    
                        #IV (Step 29/32)
                        # ---------------------------------------------------------------------------------    
                        message2print = "{(Step 29/32a) passing" + "; msg2xi-passing-to-send-by-serve-client-of-Server4Xi: " + str(msg2xi) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)            

                        send(msg2xi)
                        
                        message2print = "{(Step 29/32b) performed" + "; send-performed-by-serve-client-of-Server4Xi: " + str(send) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)             
                                        
                        
                    except Exception as e2xi:
                                    
                        # ---------------------------------------------------------------------------------    
                        #IV (Step 30/32)
                        # ---------------------------------------------------------------------------------    
                        message2print = "{(Step 30/32a) hit" + "; Exception-hit-by-serve-client-of-Server4Xi: " + " *** " + str(e2xi)  + " *** " + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                                            
                        message2print = "{(Step 30/32b) calling" + "; send-calling-by-serve-client-of-Server4Xi: " + str(send) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)       
                                        
                                                        
                         
                        send((get_unserializable_manager4xi(), repr(msg2xi))) #UNSERIALIZABLE_MANAGER4XI
    
                    # -------------------------------------------------------------------------------------    
                    #III
                    # -------------------------------------------------------------------------------------                 
                            
                # -----------------------------------------------------------------------------------------    
                #II
                # -----------------------------------------------------------------------------------------    
          
                        
                except Exception as e2xi:
    
                    # -------------------------------------------------------------------------------------    
                    #III (Step 31/32)
                    # -------------------------------------------------------------------------------------    
                    message2print = "{(Step 31/32a) hit" + "; Exception-hit-by-serve-client-of-Server4Xi: " + " *** " + str(e2xi)  + " *** " + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                                        
                    message2print = "{(Step 31/32b) hit" + "; Exception-serving-thread-hit-by-serve-client-of-Server4Xi: " + " *** " + str(threading.current_thread().name)  + " *** " + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                                                        
                    
                    info4xi('Server4Xi method serve_client: exception in thread serving %r',
                            threading.current_thread().name)
                    info4xi(' ... message was %r', msg2xi)
                    info4xi(' ... exception was %r', e2xi)
                    
                    message2print = "{(Step 31/32c) calling" + "; conn2xi-close-calling-by-serve-client-of-Server4Xi: " + str(conn2xi.close) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)       
          
                    conn2xi.close()
                    
                    message2print = "{(Step 31/32d) calling" + "; sys-exit-calling-by-serve-client-of-Server4Xi: " + str(sys.exit) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)       
     
                    sys.exit(1)
    
                    
                
                       
                # -----------------------------------------------------------------------------------------    
                #II
                # -----------------------------------------------------------------------------------------    
               
                    
            # -----------------------------------------------------------------------------------------    
            #I (Step 32/32)
            # -----------------------------------------------------------------------------------------
            message2print = "{(Step 32/32) returning" + "; yserve-client-of-Server4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                                    
                            
                                    
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    # usage:
    #     used in fallback_mapping
    #<methodize4xi method = "fallback-getvalue-of-Server4X" usage = "returns objec">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>    
    # ---------------------------------------------------------------------------------------------
    def fallback_getvalue(self, conn, ident, obj):
        '''
        Server4Xi method fallback_getvalue: returns object
        '''
        class2print = "Server4Xi"
        method2print = "fallback_getvalue"
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------         
        message2print = "{started" + "; fallback-getvalue-of-Server4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
  
                
        message2print = "{passed" + "; conn-passed-to-fallback-getvalue-of-Server4Xi: " + str(conn) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
                
        message2print = "{passed" + "; ident-passed-to-fallback-getvalue-of-Server4Xi: " + str(ident) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
                
        message2print = "{passed" + "; object-passed-to-fallback-getvalue-of-Server4Xi: " + str(obj) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
                 
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
        message2print = "{returning" + "; object-returning-by-fallback-getvalue-of-Server4Xi: " + str(obj) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                          
                  
        return obj
    #</methodize4xi>
            
        
    # ---------------------------------------------------------------------------------------------
    # fallback_str                              
    #         used in fallback_mapping
    #<methodize4xi method = "fallback-str-of-Server4X" usage = "returns object in string format">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>    
    # ---------------------------------------------------------------------------------------------
    def fallback_str(self, conn, ident, obj):
        '''
        Server4Xi method fallback_str: returns object in string format
        '''
        class2print = "Server4Xi"
        method2print = "fallback_str"
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------         
        message2print = "{started" + "; fallback-str-of-Server4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
                
        message2print = "{passed" + "; conn-passed-to-fallback-str-of-Server4Xi: " + str(conn) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
       
                
        message2print = "{passed" + "; ident-passed-to-fallback-str-of-Server4Xi: " + str(ident) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
                
        message2print = "{passed" + "; object-passed-to-fallback-str-of-Server4Xi: " + str(obj) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
              
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
        message2print = "{returning" + "; object-returning-by-fallback-str-of-Server4Xi: " + str(obj) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                                         
        
        return str(obj)
    #</methodize4xi>
            
        
    # ---------------------------------------------------------------------------------------------
    # fallback_repr                              
    #         used in fallback_mapping
    #<methodize4xi method = "fallback-repr-of-Server4X" usage = "returns object presentation">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>    
    # ---------------------------------------------------------------------------------------------
    def fallback_repr(self, conn, ident, obj):
        '''
        Server4Xi method fallback_repr: returns object presentation
        '''     
        class2print = "Server4Xi"
        method2print = "fallback_repr"
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------         
        message2print = "{started" + "; fallback-repr-of-Server4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
  
                
        message2print = "{passed" + "; conn-passed-to-fallback-repr-of-Server4Xi: " + str(conn) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
           
                
        message2print = "{passed" + "; ident-passed-to-fallback-repr-of-Server4Xi: " + str(ident) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
        
                
        message2print = "{passed" + "; object-passed-to-fallback-repr-of-Server4Xi: " + str(obj) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
           
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
        message2print = "{returning" + "; repr-object-function-returning-by-fallback-repr-of-Server4Xi: " + str(repr(obj)) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                        
        
        return repr(obj)
    #</methodize4xi>
            
    fallback_mapping = {
        '__str__':fallback_str,
        '__repr__':fallback_repr,
        '#GETVALUE':fallback_getvalue
        }

    # ---------------------------------------------------------------------------------------------
    # dummy                               
    #<methodize4xi method = "dummy-of-Server4Xi" usage = "dummy method">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>    
    # ---------------------------------------------------------------------------------------------
    def dummy(self, c):
        '''
        Server4Xi method dummy: dummy method
        '''
        class2print = "Server4Xi"
        method2print = "dummy"
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------         
        message2print = "{started" + "; dummy-of-Server4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
      
                
        message2print = "{passed" + "; connection-passed-to-dummy-of-Server4Xi: " + str(c) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
           
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
        
        
        pass 
        
        message2print = "{returning" + "; dummy-of-Server4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                        
                
                           
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    # debug_info                             
    #<methodize4xi method = "debug-info-of-Server4X" usage = "returns info useful to spot problems with refcounting">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>    
    # ---------------------------------------------------------------------------------------------
    def debug_info(self, c):
        '''
        Server4Xi method debug_info: returns info useful to spot problems with refcounting
        ''' 

        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------     
        class2print = "Server4Xi"
        method2print = "debug_info"
        
        message2print = "{started" + "; debug-info-of-Server4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
    
                
        message2print = "{passed" + "; connection-passed-to-debug-info-of-Server4Xi: " + str(c) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
   

        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
        
        self.mutex.acquire()
        
        message2print = "{acquired" + "; mutex-of-server2xi-acquired-by-debug-info-of-Server4Xi: " + str(self.mutex.acquire) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
        
        try:
            result = []
                     
         
            keys = list(self.id_to_obj.keys())
            
            keys.sort()

            
                        
            for ident in keys:
            
                message2print = "{using" + "; ident-using-by-debug-info-of-Server4Xi: " + str(ident)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                
            
                if ident != 0:
                    result.append('  %s:       refcount=%s\n    %s' %
                                  (ident, self.id_to_refcount[ident],
                                   str(self.id_to_obj[ident][0])[:75]))
                    
                    message2print = "{appended" + "; ident-appended-by-by-debug-info-of-Server4Xi: " + str(ident) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)             
                    
                    message2print = "{appended" + "; id-to-refcount-of-server2xi-appended-by-by-debug-info-of-Server4Xi: " + str(self.id_to_refcount[ident]) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)             
                    
                    message2print = "{appended" + "; id-to-object-of-server2xi-appended-by-debug-info-of-Server4Xi: " + str(self.id_to_obj[ident][0]) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)             
            
            message2print = "{created" + "; result-created-by-by-debug-info-of-Server4Xi: " + str(result) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
                                                                                    
                          
            return '\n'.join(result)
        finally:
            self.mutex.release()
            
            message2print = "{released" + "; mutex-of-server2xi-released-by-debug-info-of-Server4Xi: " + str(self.mutex.release) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
         
                        
            message2print = "{created" + "; result-created-by-debug-info-of-Server4Xi: " + str(result) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
   

                        
                
                             
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "number-of-objects-of-Server4Xi" usage = "returns number of shared objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>                            
    # ---------------------------------------------------------------------------------------------
    def number_of_objects(self, c):
        '''
        Server4Xi method number_of_objects: returns number of shared objects
        '''
        class2print = "Server4Xi"
        method2print = "number_of_objects"
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------         
        message2print = "{started" + "; number-of-objects-of-Server4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
                
        message2print = "{passed" + "; connection-passed-to-number-of-objects-of-Server4Xi: " + str(c) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
         
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
        message2print = "{returning" + "; object-quantity-returning-by-number-of-objects-of-Server4Xi: " + str(len(self.id_to_obj) - 1) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                                     
                   
        return len(self.id_to_obj) - 1      # don't count ident=0
        
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "shutdown-of-Server4X" usage = "shuts down this proces">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>                               
    # ---------------------------------------------------------------------------------------------
    def shutdown(self, c):
        '''
        Server4Xi method shutdown: shuts down this process
        '''


        # -----------------------------------------------------------------------------------------    
        # (Step 1/11)
        # -----------------------------------------------------------------------------------------         
        
        class2print = "Server4Xi"
        method2print = "shutdown"
        
        message2print = "{(Step 1/11a) started" + "; shutdown-of-Server4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
                
        message2print = "{(Step 1/11b) passed" + "; connection-passed-to-shutdown-of-Server4Xi: " + str(c) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
     
        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # -----------------------------------------------------------------------------------------            
            message2print = "{(StepOriginal) passing" + "; connection-passing-to-Server-shutdown-by-shutdown-of-Server4Xi: " + str(c) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                            
                            
            Server.shutdown(self, c)           
        else:
                
        
            
            # -----------------------------------------------------------------------------------------    
            # (Step 2/11)
            # -----------------------------------------------------------------------------------------                 
            message2print = "{(Step 2/11) trying" + "; shutdown-of-connection-trying-by-shutdown-of-Server4Xi: " + str(c)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
                            
               
            try:
                try:

                    # -------------------------------------------------------------------------------------    
                    # (Step 3/11)
                    # -------------------------------------------------------------------------------------                 
                    message2print = "{(Step 3/11a) passing" + "; manager-passing-to-connection-send-by-shutdown-of-Server4Xi: " + str(get_return_manager4xi()) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)            
                                    
                                    
                           
                    c.send((get_return_manager4xi(), None)) #RETURN_MANAGER4XI
    
                    message2print = "{(Step 3/11b) performed" + "; connection-send-performed-by-shutdown-of-Server4Xi: " + str(c.send) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)             
                                    
                      
                    
                    # -------------------------------------------------------------------------------------    
                    # (Step 4/11)
                    # -------------------------------------------------------------------------------------                 
            
                    if sys.stdout != sys.__stdout__:
                        # resetting stdout, stderr
                        
                        sys.stdout = sys.__stdout__
                        
                        message2print = "{(Step 4/11a) set" + "; sys-stdout-set-by-shutdown-of-Server4Xi to: " + str(sys.stdout)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)    
                                        
                                        
                        
                        sys.stderr = sys.__stderr__
                        
                        message2print = "{(Step 4/11b) set" + "; sys-stderr-set-by-shutdown-of-Server4Xi to: " + str(sys.stderr)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)    
      
                  
                    
                    # -------------------------------------------------------------------------------------    
                    # (Step 5/11)
                    # -------------------------------------------------------------------------------------                 
                    message2print = "{(Step 5/11a) calling" + "; run-finalizers4xi-calling-by-shutdown-of-Server4Xi: " + str(_run_finalizers4xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)       
                                    
                                    
                    
                    _run_finalizers4xi(0)
    
                    message2print = "{(Step 5/11b) performed" + "; run-finalizers4xi-performed-by-shutdown-of-Server4Xi: " + str(_run_finalizers4xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)             
                                      
     
                    # -------------------------------------------------------------------------------------    
                    # (Step 6/11)
                    # -------------------------------------------------------------------------------------                 
            
                    for p2xi in active_children4xi():
                        # terminating a child process of manager
                        
                        message2print = "{(Step 6/11a) got" + "; child-active-process-got-by-shutdown-of-Server4Xi: " + str(p2xi)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                                        
                        message2print = "{(Step 6/11b) calling" + "; child-active-process-terminate-calling-by-shutdown-of-Server4Xi: " + str(p2xi.terminate) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)       

                        p2xi.terminate()
    
                      
     
                    # -------------------------------------------------------------------------------------    
                    # (Step 7/11)
                    # -------------------------------------------------------------------------------------                 
            
                    for p2xi in active_children4xi():
                        # joining a child process
                        
                        
                        message2print = "{(Step 6/11a) got" + "; child-active-process-got-by-shutdown-of-Server4Xi: " + str(p2xi)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                                        
                        message2print = "{(Step 6/11b) calling" + "; child-active-process-join-calling-by-shutdown-of-Server4Xi: " + str(p2xi.join) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)       
                        
                        
                        p2xi.join()
    
                       
     
                    # -------------------------------------------------------------------------------------    
                    # (Step 8/11)
                    # -------------------------------------------------------------------------------------                 
            
                    _run_finalizers4xi()
                    
                    message2print = "{(Step 8/11) performed" + "; run-finalizers4xi-performed-by-shutdown-of-Server4Xi: " + str(_run_finalizers4xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)             
                  
                    info4xi('Server4Xi method shutdown: exiting with exitcode 0')

    
                except:
     
                    # -------------------------------------------------------------------------------------    
                    # (Step 9/11)
                    # -------------------------------------------------------------------------------------                 
                    
                    import traceback

                    traceback.print_exc()
    
                    message2print = "{(Step 9/11) hit" + "; exception-hit-by-shutdown-of-Server4Xi: " + " *** exception in shutdown *** " + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                                        
                                       
                     
            # -----------------------------------------------------------------------------------------    
            #
            # -----------------------------------------------------------------------------------------    
                               
            finally:
    
                # -------------------------------------------------------------------------------------    
                # (Step 10/11)
                # -------------------------------------------------------------------------------------                 
                message2print = "{(Step 10/11) calling" + "; exit-calling-by-shutdown-of-Server4Xi: " + str(exit) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)       

                exit(0)
                
                
                
            # -----------------------------------------------------------------------------------------    
            # (Step 11/11)
            # -----------------------------------------------------------------------------------------                 
            message2print = "{(Step 11/11) returning" + "; shutdown-of-Server4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                                    
                            
                                    
    #</methodize4xi>
               
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "create-of-Server4Xi" usage = "creates a new shared object and return its id"> 
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>                             
    # ---------------------------------------------------------------------------------------------
    def create(self, c, typeid, *args, **kwds):
        '''
        Server4Xi method create: creates a new shared object and return its id
        '''
    
        # -----------------------------------------------------------------------------------------    
        # (Step 1/12)
        # -----------------------------------------------------------------------------------------                 
    
        class2print = "Server4Xi"
        method2print = "create"
        
        message2print = "{(Step 1/12a) started" + "; create-of-Server4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
      
                
        message2print = "{(Step 1/12b) passed" + "; connection-passed-to-create-of-Server4Xi: " + str(c) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
                
        message2print = "{(Step 1/12c) passed" + "; typeid-passed-to-create-of-Server4Xi: " + str(typeid) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
                
        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            message2print = "{(StepOriginal) started" + "; create-of-Server-started: " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
                                    
            
            
            self.mutex.acquire()
            try:
                callable, exposed, method_to_typeid, proxytype = \
                          self.registry[typeid]
    
                if callable is None:
                    assert len(args) == 1 and not kwds
                    obj = args[0]
                else:
                    obj = callable(*args, **kwds)
    
                if exposed is None:
                    exposed = public_methods(obj)
                if method_to_typeid is not None:
                    assert type(method_to_typeid) is dict
                    exposed = list(exposed) + list(method_to_typeid)
    
                ident = '%x' % id(obj)  # convert to string because xmlrpclib
                                        # only has 32 bit signed integers
                util.debug('%r callable returned object with id %r', typeid, ident)
    
                self.id_to_obj[ident] = (obj, set(exposed), method_to_typeid)
                if ident not in self.id_to_refcount:
                    self.id_to_refcount[ident] = 0
                # increment the reference count immediately, to avoid
                # this object being garbage collected before a Proxy
                # object for it can be created.  The caller of create()
                # is responsible for doing a decref once the Proxy object
                # has been created.
                self.incref(c, ident)
                
                return ident, tuple(exposed)
            finally:
                self.mutex.release() 
                         
        else:
                
    
            # -----------------------------------------------------------------------------------------    
            # (Step 2/12)
            # -----------------------------------------------------------------------------------------                 

            self.mutex.acquire()
            
            message2print = "{(Step 2/12) acquired" + "; mutex-of-server2xi-acquired-by-ycreate-of-Server4Xi: " + str(self.mutex.acquire) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
             
            
            # -----------------------------------------------------------------------------------------    
            #
            # -----------------------------------------------------------------------------------------                 
               
            try:
                
                # -----------------------------------------------------------------------------------------    
                # (Step 3/12)
                # -----------------------------------------------------------------------------------------    
                
                callable, exposed, method_to_typeid, proxytype = \
                          self.registry[typeid]
                
                message2print = "{(Step 3/12a) set" + "; callable-set-by-ycreate-of-Server4Xi to: " + str(callable)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                
                message2print = "{(Step 3/12b) set" + "; exposed-set-by-ycreate-of-Server4Xi to: " + str(exposed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                
                message2print = "{(Step 3/12c) set" + "; method-to-typeid-set-by-ycreate-of-Server4Xi to: " + str(method_to_typeid)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                
                message2print = "{(Step 3/12d) set" + "; proxytype-set-by-ycreate-of-Server4Xi to: " + str(proxytype)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                         
                                
                
                # -----------------------------------------------------------------------------------------    
                # (Step 4/12)
                # -----------------------------------------------------------------------------------------    
        
                if callable is None:
                    assert len(args) == 1 and not kwds
                    obj = args[0]
                    
                    message2print = "{(Step 4/12a) set" + "; object-set-by-ycreate-of-Server4Xi to: " + str(obj)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                                    
                                    
                else:
                    obj = callable(*args, **kwds)
                     
                    message2print = "{(Step 4/12b) set" + "; object-set-by-ycreate-of-Server4Xi to: " + str(obj)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                       
                
   
                
                # -----------------------------------------------------------------------------------------    
                # (Step 5/12)
                # -----------------------------------------------------------------------------------------    
        
                if exposed is None:
                
                    message2print = "{(Step 5/12a) passing" + "; object-passing-to-public-methods4xi-by-ycreate-of-Server4Xi: " + str(obj) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)            
                                    
                                    
                    exposed = public_methods4xi(obj)
                
                    message2print = "{(Step 5/12b) set" + "; exposed-set-by-ycreate-of-Server4Xi to: " + str(exposed)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                                    
                                    
                
                # -----------------------------------------------------------------------------------------    
                # (Step 6/12)
                # -----------------------------------------------------------------------------------------    
                        
                if method_to_typeid is not None:
                    assert type(method_to_typeid) is dict
                    exposed = list(exposed) + list(method_to_typeid)
                
                    message2print = "{(Step 6/12) set" + "; exposed-set-by-ycreate-of-Server4Xi to: " + str(exposed)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                   
   
                
                # -----------------------------------------------------------------------------------------    
                # (Step 7/12)
                # -----------------------------------------------------------------------------------------    
        
                ident = '%x' % id(obj)  # convert to string because xmlrpclib
                                        # only has 32 bit signed integers
                                
                message2print = "{(Step 7/12) set" + "; ident-set-by-ycreate-of-Server4Xi to: " + str(ident)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                
                                
                
                # -----------------------------------------------------------------------------------------    
                # (Step 8/12)
                # -----------------------------------------------------------------------------------------    
        
                self.id_to_obj[ident] = (obj, set(exposed), method_to_typeid)
                
                message2print = "{set" + "; id-to-object-of-server2xi-set-by-ycreate-of-Server4Xi to: " + str(self.id_to_obj[ident])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                
                                
                
                # -----------------------------------------------------------------------------------------    
                # (Step 9/12)
                # -----------------------------------------------------------------------------------------    
                    
                if ident not in self.id_to_refcount:
                    self.id_to_refcount[ident] = 0
                    
                    message2print = "{(Step 9/12) set" + "; id-to-refcount-of-server2xi-set-by-ycreate-of-Server4Xi to: " + str(self.id_to_refcount[ident])  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                                    
                                    
                # increment the reference count immediately, to avoid
                # this object being garbage collected before a Proxy
                # object for it can be created.  The caller of create()
                # is responsible for doing a decref once the Proxy object
                # has been created.
                
                
   
                
                # -----------------------------------------------------------------------------------------    
                # (Step 10/12)
                # -----------------------------------------------------------------------------------------    
                message2print = "{(Step 10/12a) passing" + "; connection-passing-to-incref-of-server2xi-by-ycreate-of-Server4Xi: " + str(c) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                        
                message2print = "{(Step 10/12b) passing" + "; ident-passing-to-incref-of-server2xi-by-ycreate-of-Server4Xi: " + str(ident) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                        
                    
                self.incref(c, ident)
                
                
  
                
                # -----------------------------------------------------------------------------------------    
                # (Step 11/12)
                # -----------------------------------------------------------------------------------------    
                message2print = "{(Step 11/12a) returning" + "; ident-returning-by-ycreate-of-Server4Xi: " + str(ident) + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)    
                
                message2print = "{(Step 11/12a) returning" + "; tuple-exposed-returning-by-ycreate-of-Server4Xi: " + str(tuple(exposed)) + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)                                         
                                
                return ident, tuple(exposed)
                
                
            finally:
    
                # -----------------------------------------------------------------------------------------    
                # (Step 12/12)
                # -----------------------------------------------------------------------------------------    
                
                self.mutex.release()
                
                message2print = "{(Step 12/12) released" + "; mutex-of-server2xi-released-by-ycreate-of-Server4Xi: " + str(self.mutex.release) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                
                
       
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get-methods-of-Server4X" usage = "returns the methods of the shared object indicated by token">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>                                 
    # ---------------------------------------------------------------------------------------------
    def get_methods(self, c, token):
        '''
        Server4Xi method get_methods: returns the methods of the shared object indicated by token
        '''
        class2print = "Server4Xi"
        method2print = "get_methods"
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------         
        message2print = "{started" + "; get-methods-of-Server4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
                
        message2print = "{passed" + "; connection-passed-to-get-methods-of-Server4Xi: " + str(c) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
       
                
        message2print = "{passed" + "; token-passed-to-get-methods-of-Server4Xi: " + str(token) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
           
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
        message2print = "{returning" + "; methods-returning-by-get-methods-of-Server4Xi: " + str(tuple(self.id_to_obj[token.id][1])) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                                    
                   
        return tuple(self.id_to_obj[token.id][1])
        
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "accept-connection-of-Server4Xi" usage = "spawns a new thread to serve this connection">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>    
    # ---------------------------------------------------------------------------------------------
    def accept_connection(self, c, name):
        '''
        Server4Xi method accept_connection: spawns a new thread to serve this connection
        '''


        # -----------------------------------------------------------------------------------------    
        # (Step 1/5)
        # -----------------------------------------------------------------------------------------         

        class2print = "Server4Xi"
        method2print = "accept_connection"
        
        message2print = "{(Step 1/5a) started" + "; accept-connection-of-Server4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
                
        message2print = "{(Step 1/5b) passed" + "; connection-passed-to-accept-connection-of-Server4Xi: " + str(c) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
                
        message2print = "{(Step 1/5c) passed" + "; name-passed-to-accept-connection-of-Server4Xi: " + str(name) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
         

        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            message2print = "{(StepOriginal) passing" + "; connection-passing-to-Server-accept-connection-by-yaccept-connection-of-Server4Xi: " + str(c) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                            
            message2print = "{(StepOriginal) passing" + "; name-passing-to-Server-accept-connection-by-yaccept-connection-of-Server4Xi: " + str(name) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                              
       
            Server.accept_connection(self, c, name)           
        else:
                        
            # -----------------------------------------------------------------------------------------    
            # (Step 2/5)
            # -----------------------------------------------------------------------------------------                 
               
            threading.current_thread().name = name
            
            message2print = "{(Step 2/5) set" + "; current-thread-name-set-by-yaccept-connection-of-Server4Xi to: " + str(threading.current_thread().name)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                            
                            
            
            # -----------------------------------------------------------------------------------------    
            # (Step 3/5)
            # -----------------------------------------------------------------------------------------                 
            message2print = "{(Step 3/5a) passing" + "; return-manager4xi-passing-to-connection-send-by-yaccept-connection-of-Server4Xi: " + str(get_return_manager4xi()) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                            
                            
            
            c.send((get_return_manager4xi(), None))
            
            message2print = "{(Step 3/5b) performed" + "; connection-send-performed-by-yaccept-connection-of-Server4Xi: " + str(c.send) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                            
            
            # -----------------------------------------------------------------------------------------    
            # (Step 4/5)
            # -----------------------------------------------------------------------------------------                 
            message2print = "{(Step 4/5) passing" + "; connection-passing-to-serve-client-of-server2xi-by-yaccept-connection-of-Server4Xi: " + str(c) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                            
                            
            
               
            self.serve_client(c)
            
      
            
            # -----------------------------------------------------------------------------------------    
            # (Step 5/5)
            # ----------------------------------------------------------------------------------------- 
            message2print = "{(Step 5/5) returning" + "; yaccept-connection-of-Server4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                                    
                            
                                    
                            

    #</methodize4xi>
               
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "incref-of-Server4Xi" usage = "increases ref with (self, c, ident)">                           
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>    
    # ---------------------------------------------------------------------------------------------
    def incref(self, c, ident):
        '''
        Server4Xi method incref: increases ref with (self, c, ident) 
        '''

 
        
        class2print = "Server4Xi"
        method2print = "incref"

        # -----------------------------------------------------------------------------------------    
        # (Step 1/4)
        # -----------------------------------------------------------------------------------------         
             
        message2print = "{(Step 1/4a) started" + "; incref-of-Server4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
                
        message2print = "{(Step 1/4b) passed" + "; connection-passed-to-incref-of-Server4Xi: " + str(c) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
                
        message2print = "{(Step 1/4c) passed" + "; ident-passed-to-incref-of-Server4Xi: " + str(ident) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
               
        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            message2print = "{(StepOriginal) passing" + "; connection-passing-to-Server-incref-by-yincref-of-Server4Xi: " + str() + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            
            message2print = "{(StepOriginal) passing" + "; ident-passing-to-Server-incref-by-yincref-of-Server4Xi: " + str(ident) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                 
                            
            Server.incref(self, c, ident)
            
        else:            
            # -----------------------------------------------------------------------------------------    
            # (Step 2/4)
            # -----------------------------------------------------------------------------------------                 
               
            self.mutex.acquire()
    
            message2print = "{(Step 2/4) acquired" + "; mutex-of-server2xi-acquired-by-yincref-of-Server4Xi: " + str(self.mutex.acquire) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
               
            
            # -----------------------------------------------------------------------------------------    
            # (Step 3/4)
            # -----------------------------------------------------------------------------------------                 
               
            try:
                self.id_to_refcount[ident] += 1
                
                message2print = "{(Step 3/4a) set" + "; id-to-refcountod-server2xi-set-by-yincref-of-Server4Xi to: " + str(self.id_to_refcount[ident])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                
                                
                
            finally:
                self.mutex.release() 
                
                message2print = "{(Step 3/4b) released" + "; mutex-of-server2xi-released-by-yincref-of-Server4Xi: " + str(self.mutex.release) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                       

                        
            # -----------------------------------------------------------------------------------------    
            # (Step 4/4)
            # -----------------------------------------------------------------------------------------                 
            
            message2print = "{(Step 4/4) returning" + "; yincref-of-Server4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                                    
                            
                                    
    #</methodize4xi>
               
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "decref-of-Server4Xi" usage = "decreases ref with (self, c, ident)">                          
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>    
    # ---------------------------------------------------------------------------------------------
    def decref(self, c, ident):
        '''
        Server4Xi method decref: decreases ref with (self, c, ident) 
        '''


        # -----------------------------------------------------------------------------------------    
        # (Step 1/4)
        # -----------------------------------------------------------------------------------------         

        class2print = "Server4Xi"
        method2print = "decref"
        
        message2print = "{(Step 1/4a) started" + "; decref-of-Server4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
                
        message2print = "{(Step 1/4b) passed" + "; connection-passed-to-decref-of-Server4Xi: " + str(c) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
                
        message2print = "{(Step 1/4c) passed" + "; ident-passed-to-decref-of-Server4Xi: " + str(ident) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      

        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            message2print = "{(StepOriginal) passing" + "; connection-passing-to-Server-decref-by-ydecref-of-Server4Xi: " + str(c) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            
            message2print = "{(StepOriginal) passing" + "; ident-passing-to-Server-decref-by-ydecref-of-Server4Xi: " + str(ident) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                             
                            
            Server.decref(self, c, ident)
            
        else:            
        
            
            # -----------------------------------------------------------------------------------------    
            # (Step 2/4)
            # -----------------------------------------------------------------------------------------                 
               
            self.mutex.acquire()
    
            message2print = "{(Step 2/4) acquired" + "; mutex-of-server2xi-acquired-by-ydecref-of-Server4Xi: " + str(self.mutex.acquire) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
 
            
            # -----------------------------------------------------------------------------------------    
            # (Step 3/4)
            # -----------------------------------------------------------------------------------------                 
              
            try:
                assert self.id_to_refcount[ident] >= 1
                
                message2print = "{(Step 3/4a) asserted" + "; id-to-refcount-of-server2xi-asserted-by-ydecref-of-Server4Xi: " + str(self.id_to_refcount[ident]) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                
                
                self.id_to_refcount[ident] -= 1
                
                message2print = "{(Step 3/4b) set" + "; id-to-refcount-of-server2xi-set-by-ydecref-of-Server4Xi to: " + str(self.id_to_refcount[ident])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                
                                
                
                if self.id_to_refcount[ident] == 0:
                    
                    message2print = "{(Step 3/4c) deleting" + "; object-of-server2xi-deleting-by-ydecref-of-Server4Xi: " + str(self.id_to_obj[ident]) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                     
                    
                    message2print = "{(Step 3/4c) deleting" + "; id-to-refcount-of-server2xi-deleting-by-ydecref-of-Server4Xi: " + str(self.id_to_refcount[ident]) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                  
                
                    del self.id_to_obj[ident], self.id_to_refcount[ident]
                    
                    
                    # disposing an object
                    
     
            finally:
                self.mutex.release()
                
                message2print = "{released" + "; mutex-of-server2xi-released-by-ydecref-of-Server4Xi: " + str(self.mutex.release) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                
                
           
            
            # -----------------------------------------------------------------------------------------    
            # (Step 4/4)
            # -----------------------------------------------------------------------------------------                 

    #</methodize4xi>
                              
                                       
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-Server4X" usage = "senses Server4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>    
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses Server4Xi class objects
        '''

        
        class2print = 'Server4Xi'
        method2print = 'sense'
        
        
        message2print = "{started" + "; sense-of-Server4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
  
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        message2print = "{sensing" + "; Server4Xi-sensing-by-sense-of-Server4Xi: " + str(Server4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                     
        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb2xi = sense_object4xi(self)
        
        message2print = "{got" + "; kb2xi-got-by-sense-of-Server4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                

                                  
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # -----------------------------------------------------------------------------------------         
        try:
            kb2xi['public-of-Server4Xi'] = Server4Xi.public
            kb2xi['registry-of-server2xi'] = self.registry
            kb2xi['authkey-of-server2xi'] = self.authkey  
            kb2xi['listener-of-server2xi'] = self.listener  
            kb2xi['address-of-server2xi'] = self.address
            kb2xi['id-to-obj-of-server2xi'] = self.id_to_obj  
            kb2xi['id-to-refcount-of-server2xi'] = self.id_to_refcount    
            kb2xi['mutex-of-server2xi'] = self.mutex
            kb2xi['stop-of-server2xi'] = self.stop  
                                                           
        except:
            message2print = "{yielded" + "; kb2xi-yielded-by-sense-of-Server4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
    

        
        
      
        message2print = "{sensed" + "; Server4Xi-sensed-by-sense-of-Server4Xi: " + str(Server4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        message2print = "{returning" + "; kb2xi-returning-by-sense-of-Server4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                                   
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-Server4Xi" usage = "reasons Server4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>    
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons Server4Xi class objects
        '''
    
        
        class2print = 'Server4Xi'
        method2print = 'reason'

        
        message2print = "{started" + "; reason-of-Server4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{reasoning" + "; Server4Xi-reasoning-by-reason-of-Server4Xi: " + str(Server4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
                
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        kb4xi = reason_object4xi(self)
        
        
        message2print = "{got" + "; kb4xi-got-by-reason-of-Server4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        
        
          
        
        message2print = "{reasoned" + "; Server4Xi-reasoned-by-reason-of-Server4Xi: " + str(Server4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb4xi-returning-by-reason-of-Server4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                                  
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-Server4Xi" usage = "aces Server4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>    
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces Server4Xi class objects 
        '''
 
        
        
        class2print = 'Server4Xi'
        method2print = 'ace'
        
        
        message2print = "{started" + "; ace-of-Server4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
              
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{acing" + "; Server4Xi-acing-by-ace-of-Server4Xi: " + str(Server4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                       
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{got" + "; kb8xi-got-by-ace-of-Server4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                

        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        
          
        message2print = "{aced" + "; Server4Xi-aced-by-ace-of-Server4Xi: " + str(Server4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb8xi-returning-by-ace-of-Server4X: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                               
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
#</class4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_server4xi.txt
#   sense-test-server4xi.xml
#   reason-test-server4xi.xml
#   ace-test-server4xi.xml
#"
#>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of server4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>

